import typer
import yaml
import pathlib
from git.repo import Repo
from jinja2 import Environment, FileSystemLoader

from rich import print as rprint

import questionary

from c6t.configure.credentials import ContrastAPICredentials
from c6t.ui.auth import ContrastUIAuthManager
from c6t.api.agent_config import AgentConfig

from c6t.external.integrations.scw.contrast_scw import scw_create, scw_delete

app = typer.Typer()
integrations_app = typer.Typer()
scw_integration_app = typer.Typer()
app.add_typer(
    integrations_app, name="integrations", help="Integrations with other tools"
)
integrations_app.add_typer(scw_integration_app, name="scw", help="Secure Code Warrior")


@app.command("login")
def login(profile: str = "default") -> None:
    """
    Login to the Contrast platform using your UI credentials (username/password).
    This will automatically configure your API credentials and save them to the
    credentials file.
    """

    contrast_environment = questionary.select(
        "Select your Contrast TeamServer Environment:",
        choices=[
            {
                "name": "Free Trial",
                "value": "https://cs004.contrastsecurity.com/Contrast",
            },
            {
                "name": "Evaluation",
                "value": "https://eval.contrastsecurity.com/Contrast",
            },
            {"name": "Enterprise", "value": "Enterprise"},
        ],
        default={
            "name": "Free Trial",
            "value": "https://cs004.contrastsecurity.com/Contrast",
        },
    ).ask()

    if contrast_environment == "Enterprise":
        contrast_environment = questionary.text(
            "Enter your Contrast TeamServer URL:"
        ).ask()

    if contrast_environment:
        auth = ContrastUIAuthManager(base_url=contrast_environment)
        auth.login(profile=profile)


@app.command("configure")
def configure(profile: str = "default") -> None:
    """
    Configure the c6t CLI credentials.
    This will save your credentials to the credentials file.
    """
    credentials = ContrastAPICredentials()
    credentials.profile = profile
    credentials.base_url = typer.prompt("Contrast TeamServer URL")
    credentials.username = typer.prompt("Contrast username")
    credentials.service_key = typer.prompt("Contrast service key")
    credentials.api_key = typer.prompt("Contrast API key")
    credentials.organization_uuid = typer.prompt("Contrast organization UUID")
    credentials.superadmin = typer.confirm("Contrast superadmin status")
    credentials.write_credentials_to_file()


@app.command("agent-config")
def agent_config(
    profile: str = "default",
    path: str = "contrast_security.yaml",
    language: str = "JAVA",
) -> None:
    """
    Gets the Contrast Agent YAML config file from TeamServer
    and saves it to the current working directory.
    """

    # Get initials from user input
    initials = typer.prompt("Enter your initials")
    environment = "dev"

    # Attach to repo in current working directory
    repo = Repo(".")

    # Get credentials from file
    rprint("Getting agent config from TeamServer...")
    agent_config = AgentConfig(profile=profile)
    raw_yaml_text = agent_config.get_agent_credentials(language=language)
    credentials = yaml.safe_load(raw_yaml_text).get("api")

    # Load Jinga2 template and render using YAML text
    rprint("Rendering agent config...")
    template_path = pathlib.Path("~/.c6t/templates").expanduser()
    template_loader = FileSystemLoader(searchpath=template_path)
    template_env = Environment(loader=template_loader)
    template = template_env.get_template("contrast_security.yaml.j2")
    rendered_yaml_text = template.render(
        url=credentials.get("url"),
        api_key=credentials.get("api_key"),
        service_key=credentials.get("service_key"),
        user_name=credentials.get("user_name"),
        application_name=f"TerracottaBank-{initials}",
        branch_name=repo.active_branch,
        commit_hash=repo.head.commit,
        committer=repo.head.commit.author,
        repository=repo.remotes.origin.url,
        environment=environment,
    )

    rprint("Writing agent config to file...")
    agent_config.write_agent_config_to_file(path=path, yaml_text=rendered_yaml_text)


# TODO: Add a command to populate vulnerability references with Secure Code Warrior links
@scw_integration_app.command("create")
def scw(profile: str = "default") -> None:
    """
    Populate vulnerability references with Secure Code Warrior links.
    """
    print("Creating SCW links...")
    scw_create(profile=profile)


@scw_integration_app.command("delete")
def scw_reset(profile: str = "default") -> None:
    """
    Delete vulnerability references and reset to the original Contrast links.
    """
    print("Deleting SCW links...")
    scw_delete(profile=profile)


if __name__ == "__main__":
    app()



# Class for interacting with the Contrast TeamServer REST APIs
# Author: josh.anderson@contrastsecurity.com / jonathan.harper@contrastsecurity.com

import json
import re
import base64
from urllib.request import Request, urlopen

from typing import Any, Dict, List

from c6t.configure.credentials import ContrastAPICredentials


def load_config():
    with open("config.json", "r") as config:
        config = json.load(config)

    return config


def contrast_instance_from_json(credentials: ContrastAPICredentials):
    return ContrastTeamServer(
        credentials.base_url,
        credentials.api_key,
        _create_authorization_header(credentials),
    )


def _create_authorization_header(credentials: ContrastAPICredentials) -> str:
    """
    Create the Authorization header for the TeamServer API
    """
    unencoded_authorization_header = f"{credentials.username}:{credentials.service_key}"
    encoded_authorization_header = base64.b64encode(
        unencoded_authorization_header.encode("utf-8")
    )
    return encoded_authorization_header.decode("utf-8")


class ContrastTeamServer:

    def __init__(
        self,
        teamserver_url: str,
        api_key: str,
        authorization_header: str,
        application_metadata_field_name: str = "",
    ):
        teamserver_url = teamserver_url.strip()

        regexBaseUrl = "^(http|https):\\/\\/[a-z0-9\\.]*(:)?([0-9]*)"

        if re.match(regexBaseUrl, teamserver_url):
            if re.match(regexBaseUrl + "$", teamserver_url):
                teamserver_url = teamserver_url + "/Contrast/api/ng/"
            elif re.match(regexBaseUrl + "\\/$", teamserver_url):
                teamserver_url = teamserver_url + "Contrast/api/ng/"
            elif re.match(regexBaseUrl + "\\/Contrast$", teamserver_url):
                teamserver_url = teamserver_url + "/api/ng/"
            elif re.match(regexBaseUrl + "\\/Contrast\\/$", teamserver_url):
                teamserver_url = teamserver_url + "api/ng/"
            elif re.match(regexBaseUrl + "\\/Contrast\\/api$", teamserver_url):
                teamserver_url = teamserver_url + "/ng/"
            elif re.match(regexBaseUrl + "\\/Contrast\\/api\\/$", teamserver_url):
                teamserver_url = teamserver_url + "ng/"
            elif re.match(regexBaseUrl + "\\/Contrast\\/api\\/ng$", teamserver_url):
                teamserver_url = teamserver_url + "/"
            elif (
                re.match(regexBaseUrl + "\\/Contrast\\/api\\/ng\\/$", teamserver_url)
                == None
            ):
                raise ValueError("Unrecognised TeamServer URL")

        self._teamserver_url: str = teamserver_url
        self._api_key: str = api_key
        self._authorization_header: str = authorization_header
        self._application_metadata_field_name: str = application_metadata_field_name

        self._is_superadmin: bool = False

        self._title_cwe_cache: Dict[Any, Any] = {}

    @property
    def teamserver_url(self):
        return self._teamserver_url

    # Function to call the Contrast TeamServer REST API and retrieve results as JSON
    def api_request(self, path: str, api_key: str = ""):
        if not api_key:
            api_key = self._api_key

        req = Request(self._teamserver_url + path)
        req.add_header("Accept", "application/json")
        req.add_header("Api-Key", api_key)
        req.add_header("Authorization", self._authorization_header)

        res = urlopen(req).read()
        data = json.loads(res.decode("utf-8"))

        return data

    # Function to POST data to the Contrast TeamServer REST API and retrieve results as JSON
    def post_api_request(self, path: str, data: bytes, api_key: str = ""):
        if not api_key:
            api_key = self._api_key

        req = Request(self._teamserver_url + path, data)
        req.add_header("Content-Type", "application/json")
        req.add_header("Accept", "application/json")
        req.add_header("Api-Key", api_key)
        req.add_header("Authorization", self._authorization_header)

        res = urlopen(req).read()
        data = json.loads(res.decode("utf-8"))

        return data

    # Superadmin API call to retrieve the API key for a specific organization
    def org_api_key(self, org_id: str):
        if self._is_superadmin:
            return self.api_request("superadmin/organizations/" + org_id + "/apiKey")
        else:
            return {"api_key": self._api_key}

    # Organization specific API call to list assess policy (rules)
    def list_org_policy(self, org_id: str, api_key: str, expand_apps: bool = False):
        call = org_id + "/rules?expand=skip_links"
        if expand_apps:
            call += ",app_assess_rules"

        return self.api_request(call, api_key)["rules"]

    # Organization specific API call to retrieve CWE ID for a trace. Cache results locally by rule name as a speedup.
    def trace_cwe(self, org_id: str, title: str, api_key: str):
        if len(self._title_cwe_cache) == 0:
            policies = self.list_org_policy(org_id, api_key)
            for policy in policies:
                self._title_cwe_cache[policy["title"]] = (
                    policy["cwe"].split("/")[-1].replace(".html", "")
                )

        return self._title_cwe_cache[title]

    def update_rule_references(
        self, org_id: str, rule_name: str, references: List[str], api_key: str
    ) -> bytes:
        values = {"references": references}

        data = json.dumps(values).encode("utf-8")

        response = self.post_api_request(org_id + "/rules/" + rule_name, data, api_key)

        return response

    def send_usage_event(self, org_id: str, is_reset: bool, api_key: str):
        usage_mode_endpoint = "undo" if is_reset else "setup"

        values = {"type": usage_mode_endpoint}

        data = json.dumps(values).encode("utf-8")

        response = self.post_api_request(org_id + "/integrations/diagnostics/scw", data)

        return response



import requests
import base64
from pathlib import Path

from c6t.configure.credentials import ContrastAPICredentials


class AgentConfig:
    def __init__(self, profile: str):
        self.session = requests.Session()
        self.credentials = ContrastAPICredentials()
        self.credentials.get_credentials_from_file(profile=profile)

        self.session.headers.update(
            {
                "API-Key": self.credentials.api_key,
                "Authorization": self._create_authorization_header(),
            }
        )

    def _create_authorization_header(self) -> str:
        """
        Create the Authorization header for the TeamServer API
        """
        unencoded_authorization_header = (
            f"{self.credentials.username}:{self.credentials.service_key}"
        )
        encoded_authorization_header = base64.b64encode(
            unencoded_authorization_header.encode("utf-8")
        )
        return encoded_authorization_header.decode("utf-8")

    def get_agent_credentials(self, language: str) -> str:
        """
        Get the Agent YAML config from the TeamServer
        """
        url = (
            f"{self.credentials.base_url}/api/ng/"
            f"{self.credentials.organization_uuid}/agents/external/default/"
            f"{language}"
        )
        response = self.session.post(url)
        if response.status_code == 200:
            return response.text
        else:
            response.raise_for_status()
        return ""

    def write_agent_config_to_file(self, yaml_text: str, path: str) -> None:
        """
        Write the Agent Config to the file system
        """
        config_path = Path(path)
        with open(config_path, "w") as f:
            f.write(yaml_text)



# Script to query Contrast TeamServer to populate the vulnerability references with Secure Code Warrior video links
# Author(s): josh.anderson@contrastsecurity.com / david.archer@contrastsecurity.com / jonathan.harper@contrastsecurity.com

from urllib.request import Request, urlopen
from c6t.external.integrations.scw.contrast_api import contrast_instance_from_json
import traceback
import json
import urllib.parse
import urllib.error

from typing import List

from c6t.configure.credentials import ContrastAPICredentials


def get_scw_base_url(cwe: str):
    return (
        "https://integration-api.securecodewarrior.com/api/v1/trial?Id=contrast&MappingList=cwe&MappingKey="
        + cwe
    )


def get_scw_data(url: str):
    try:
        req = Request(url)

        res = urlopen(req).read()
        data = json.loads(res.decode("utf-8"))

        return data
    except urllib.error.HTTPError as err:
        print(err)


def map_contrast_lang_to_scw_lang(contrast_lang: str):
    langs = {
        ".NET": "c#",
        ".NET Core": "c#(.net):mvc",
        "Java": "java",
        "Node": "nodejs",
        "Python": "python:django",
        "Ruby": "ruby",
    }  # 'Go': 'go'
    return langs.get(contrast_lang, "")


def scw_create(profile: str) -> None:
    credentials = ContrastAPICredentials()
    credentials.get_credentials_from_file(profile=profile)

    org_id = credentials.organization_uuid
    contrast = contrast_instance_from_json(credentials)

    allow_product_usage_analytics = False
    enable_verbose_error_logging = False
    org_key = credentials.api_key

    # Loop through all the Assess rules
    for rule in contrast.list_org_policy(org_id, org_key):
        print("Processing rule: " + rule["name"] + "/" + rule["title"])

        refs: List[str] = []
        video = ""

        # Extract the CWE number for this rule
        reserves = {
            "escape-templates-off": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "clickjacking-control-missing": "https://media.securecodewarrior.com/v2/Module_25_CLICKJACKING_v2.mp4",
            "event-validation-disabled": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "forms-auth-protection": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "forms-auth-redirect": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "http-only-disabled": "https://media.securecodewarrior.com/v2/Module_74_WEAK_SESSION_TOKEN_GENERATION_v2.mp4",
            "httponly": "https://media.securecodewarrior.com/v2/Module_74_WEAK_SESSION_TOKEN_GENERATION_v2.mp4",
            "max-request-length": "https://media.securecodewarrior.com/v2/Module_54_DoS_Generic_v2.mp4",
            "rails-http-only-disabled": "https://media.securecodewarrior.com/v2/Module_74_WEAK_SESSION_TOKEN_GENERATION_v2.mp4",
            "reflected-xss": "https://media.securecodewarrior.com/v2/Module_73_Reflected_Cross+Site+Scripting_v2.mp4",
            "request-validation-disabled": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "request-validation-control-disabled": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "role-manager-protection": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "session-rewriting": "https://media.securecodewarrior.com/v2/module_136_exposed_session_tokens.mp4",
            "session-regenerate": "https://media.securecodewarrior.com/v2/Module_74_WEAK_SESSION_TOKEN_GENERATION_v2.mp4",
            "stored-xss": "https://media.securecodewarrior.com/v2/Module_72_Stored_Cross+Site+Scripting_v2.mp4",
            "version-header-enabled": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "verb-tampering": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "viewstate-mac-disabled": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "wcf-detect-replays": "https://media.securecodewarrior.com/v2/Security_Misconfiguration_v2.mp4",
            "wcf-exception-details": "https://media.securecodewarrior.com/v2/module_184_error_details.mp4",
            "wcf-metadata-enabled": "https://media.securecodewarrior.com/v2/module_184_error_details.mp4",
            "x-powered-by-header": "https://media.securecodewarrior.com/v2/module_184_error_details.mp4",
            "xxssprotection-header-disabled": "https://media.securecodewarrior.com/v2/Module_73_Reflected_Cross+Site+Scripting_v2.mp4",
        }

        cwe = contrast.trace_cwe(org_id, rule["title"], org_key)

        # Compose a url for this training exercise
        scw_url = get_scw_base_url(cwe)

        # Get the data for this exercise from scw
        response = get_scw_data(scw_url)

        # We got a hit
        if response:
            # Do we have a video, if so add it?
            if "videos" in response and len(response["videos"]) > 0:
                file = response["videos"][0].replace(" ", "+")
                print("Video found: " + file)
            else:
                file = reserves.get(rule["name"], "")
        else:
            print(
                "No CWE videos received from secure code warrior: "
                + rule["name"]
                + "/"
                + rule["title"]
                + ", cwe "
                + cwe
                + ", SCW url: "
                + scw_url
            )
            file = reserves.get(rule["name"], "")

        if file != "":
            video = (
                "<br>Watch a video on this topic with Secure Code Warrior (beta):<br>"
                + file
            )
        else:
            print("Missing video cwe " + cwe + ", SCW url: " + scw_url)

        # Loop through all the languages for this rule
        for lang in rule["languages"]:

            # Map the contrast language to a SCW language
            scw_lang = map_contrast_lang_to_scw_lang(lang)

            if scw_lang != "":
                # Compose the URL for training exercise
                training_url = (
                    scw_url
                    + "&LanguageKey="
                    + urllib.parse.quote(lang)
                    + "&redirect=true"
                )

                # If this is the first language, add some chrome:
                if len(refs) == 0:
                    ref = (
                        "<br>Complete a training exercise on this topic for your language using Secure Code Warrior (beta):<br><b>"
                        + lang
                        + "</b>: "
                        + training_url
                    )
                else:
                    ref = "<b>" + lang + "</b>: " + training_url

                refs.append(ref)

        if video != "":
            refs.insert(0, video)

        if len(refs) > 0:
            res = contrast.update_rule_references(org_id, rule["name"], refs, org_key)

            if res["success"] == True: # type: ignore
                print(rule["name"] + "/" + rule["title"] + " updated successfully")
        else:
            print(
                "\n[WARNING] "
                + rule["name"]
                + "/"
                + rule["title"]
                + " no references added"
            )

        print("---")

        if allow_product_usage_analytics:
            try:
                contrast.send_usage_event(org_id, False, org_key)

            except urllib.error.HTTPError as err:
                if enable_verbose_error_logging:
                    print(err)
                    print(traceback.format_exc())
                else:
                    print("Unable to send usage data")


def scw_delete(profile: str) -> None:

    credentials = ContrastAPICredentials()
    credentials.get_credentials_from_file(profile=profile)

    org_id = credentials.organization_uuid
    contrast = contrast_instance_from_json(credentials)

    allow_product_usage_analytics = False
    enable_verbose_error_logging = False
    org_key = credentials.api_key

    for rule in contrast.list_org_policy(org_id, org_key):
        print("Processing rule: " + rule["name"] + "/" + rule["title"])

        # The reset argument has been passed, erase all rule references.
        res = contrast.update_rule_references(org_id, rule["name"], [], org_key)

        if res["success"] == True: # type: ignore
            print(rule["title"] + " reset successfully")

    if allow_product_usage_analytics:
        try:
            contrast.send_usage_event(org_id, True, org_key)

        except urllib.error.HTTPError as err:
            if enable_verbose_error_logging:
                print(err)
                print(traceback.format_exc())
            else:
                print("Unable to send usage data")



import sys

import requests

from typing import Any, List, Dict, Optional

import typer

import questionary

from c6t.configure.credentials import ContrastAPICredentials, ContrastUICredentials


class ContrastUIAuthManager:
    def __init__(self, base_url: str):
        self.ui_credentials = ContrastUICredentials()
        self.api_credentials = ContrastAPICredentials()

        self.base_url = base_url
        self.superadmin = False
        self.sso_enabled = False
        self.license_active = False
        self.two_step_verification_enabled = False
        self.session = requests.Session()

    def login(self, profile: str) -> None:
        """
        This is the main login function. It calls all the other functions
        """
        self.get_session_cookie()
        self.ui_credentials.get_username_from_user_input()
        self.check_sso(self.ui_credentials.username)
        if self.sso_enabled:
            SystemExit("SSO is enabled. Exiting.")
        self.check_if_license_is_active()
        if not self.license_active:
            SystemExit("TeamServer license is not active. Exiting.")
        self.ui_credentials.get_password_from_user_input()
        self.authenticate(self.ui_credentials.username, self.ui_credentials.password)
        self.set_xsrf_token_header()
        if self.two_step_verification_enabled:
            self.ui_credentials.get_code_from_user_input()
            self.authorize(self.ui_credentials.code)

        self.get_profile_roles()
        if self.ui_credentials.superadmin:
            while True:
                toggle_superadmin = typer.prompt("Toggle Superadmin? (y/n)")
                if toggle_superadmin == "y":
                    self.toggle_superadmin()
                    self.get_superadmin_users_roles()
                    self.organization_uuid = self.get_superadmin_organizations()
                    # TODO: SuperAdmin workflow
                    print("SuperAdmin workflow not implemented yet.")
                    break
                elif toggle_superadmin == "n":
                    print("Not using Superadmin.")
                    break
                else:
                    print("Invalid input.")

        organizations = self.get_organizations()
        self.organization_uuid = self.select_organization(organizations)

        self.api_credentials.profile = profile
        self.api_credentials.base_url = self.base_url
        self.api_credentials.username = self.ui_credentials.username
        self.api_credentials.api_key = self.get_api_key(self.organization_uuid)
        self.api_credentials.service_key = self.get_service_key()
        self.api_credentials.organization_uuid = self.organization_uuid
        self.api_credentials.superadmin = self.superadmin

        self.api_credentials.write_credentials_to_file()

    def get_session_cookie(self) -> None:
        """
        This is a hack to get the SESSION cookie set. It isn't
        completely clear why this is neccessary. The cookie is set by the
        server in the response to the request. allow_redirects is set to
        False to prevent receiving the forwarded request. If this is not
        done, a 401 error is recieved and the SESSION cookie is not set.
        """
        try:
            url = f"{self.base_url}/api/ng/agent/feature"
            response = self.session.get(url, allow_redirects=False)
            if response.status_code != 302:
                # If the response is not a 302, the SESSION cookie is not set.
                response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def get_information(self) -> None:
        """
        This gets the public information about the server.
        """
        try:
            url = f"{self.base_url}/api/public/ng/information"
            response = self.session.get(url)
            if response.status_code == 200:
                data = response.json()
                if data.get("success"):
                    print(data.get("messages")[0])
                else:
                    print(data.get("messages")[0])
                    sys.exit(1)
            else:
                response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def check_sso(self, email: str) -> bool:
        """
        This checks if the email address is associated with an SSO account.
        If the response is 200, the email address is associated with an SSO account.
        If the response is 404, the email address is not associated with an SSO account.
        If the response is anything else, raises an exception.
        """
        try:
            url = f"{self.base_url}/api/public/ng/saml/email/{email}"
            response = self.session.get(url)
            if response.status_code == 200:
                self.sso_enabled = True
            elif response.status_code == 404:
                self.sso_enabled = True
                data = response.json()
                if data.get("success"):
                    print(data.get("messages")[0])
            else:
                response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)
        return False

    def check_if_license_is_active(self) -> None:
        """
        This checks if the TeamServer license is active.
        If the response is 200, the license is active.
        If the response is anything else, raises an exception.
        """
        try:
            url = f"{self.base_url}/api/ng/contrast/license/active"
            response = self.session.head(url)
            if response.status_code == 200:
                self.license_active = True
            else:
                response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def authenticate(self, username: str, password: str) -> None:
        try:
            url = f"{self.base_url}/authenticate.html"
            data = {
                "ui": False,
                "username": username,
                "password": password,
                "sso": False,
            }
            response = self.session.post(url, data=data)
            if response.status_code == 200:
                data = response.json()
                if data.get("success"):
                    print(data.get("messages")[0])
                    if data.get("toggle_enabled"):
                        # Two-Step Verification (TSV) is enabled
                        self.two_step_verification_enabled = True
                    else:
                        # Two-Step Verification (TSV) is not enabled
                        self.two_step_verification_enabled = False
                else:
                    print(data.get("messages")[0])
                    sys.exit(1)
            else:
                response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def set_xsrf_token_header(self) -> None:
        """
        This sets the X-XSRF-TOKEN header to the value of the XSRF-TOKEN cookie.
        """
        for cookie in self.session.cookies:
            if cookie.name == "XSRF-TOKEN" and cookie.value:
                self.session.headers.update(
                    {
                        "X-XSRF-TOKEN": cookie.value,
                    }
                )

    def authorize(self, code: str) -> None:
        """
        This authorizes the user with the Two-Step Verification (TSV) code.
        """
        try:
            url = f"{self.base_url}/api/ng/tsv/authorize"
            data = {
                "code": code,
            }
            response = self.session.post(url, json=data)
            if response.status_code == 200:
                data = response.json()
                if data.get("success"):
                    print(data.get("messages")[0])
            else:
                response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def get_profile_roles(self) -> None:
        """
        This gets the roles for the user.
        """
        try:
            url = f"{self.base_url}/api/ng/profile/roles"
            response = self.session.get(url)
            if response.status_code == 200:
                data = response.json()
                if data.get("success"):
                    print(data.get("messages")[0])
                    if data.get("adminRole") == "SUPERADMIN":
                        self.ui_credentials.superadmin = True
                    else:
                        self.ui_credentials.superadmin = False
                else:
                    print(data.get("messages")[0])
                    sys.exit(1)
            else:
                response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def get_api_key(self, organization_id: str) -> str:
        """
        This gets the API key for the user.
        """
        try:
            url = f"{self.base_url}/api/ng/{organization_id}/users/keys/apikey"
            response = self.session.get(url)
            if response.status_code == 200:
                data = response.json()
                return data.get("api_key")
            else:
                response.raise_for_status()
            return ""
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def get_service_key(self) -> str:
        """
        This gets the Service key for the user.
        """
        try:
            url = f"{self.base_url}/api/ng/profile/servicekey"
            response = self.session.get(url)
            if response.status_code == 200:
                data = response.json()
                return data.get("service_key")
            else:
                response.raise_for_status()
            return ""
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def get_organizations(self) -> List[str]:
        """
        Get the organizations from TeamServer
        """
        url = f"{self.base_url}/api/ng/profile/organizations"

        try:
            response = self.session.get(url)
            if response.status_code == 200:
                data = response.json()
                if data.get("count") > 0:
                    return data.get("organizations")
                else:
                    print("No organizations found")
                    sys.exit(1)
            else:
                response.raise_for_status()
            return [""]
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def select_organization(self, organizations: Any) -> Any:
        """
        Ask the user to select an organization by index number
        """

        organization_choices: List[Dict[str, Optional[str]]] = []

        for organization in organizations:
            organization_choices.append(
                {
                    "name": organization.get("name"),
                    "value": organization.get("organization_uuid"),
                }
            )

        if len(organization_choices) == 1:
            return organization_choices[0].get("value")

        organization_uuid = questionary.select(
            message="Select your organization:",
            choices=organization_choices,
        ).ask()

        return organization_uuid

    def toggle_superadmin(self) -> None:
        """
        Toggle the SuperAdmin role for the user.
        This is a hack to get the Contrast UI key cookies set.
        Cookies are set as a side-effect of this request.
        """
        url = f"{self.base_url}/api/ng/profile/toggle"
        try:
            response = self.session.get(url)
            if response.status_code != 200:
                response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def get_superadmin_users_roles(self) -> List[str]:
        # TODO: This may not be needed.
        # TODO: Determine the meaning of the returned roles.
        """
        Get the SuperAdmin roles for the user
        """
        url = f"{self.base_url}/api/ng/superadmin/users/roles"
        try:
            response = self.session.get(url)
            if response.status_code == 200:
                data = response.json()
                roles = data.get("roles")
                print(roles)
                return roles
            else:
                response.raise_for_status()
            return [""]
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def get_superadmin_organizations(self) -> str:
        """
        Get the SuperAdmin organizations for the user
        """
        url = (
            f"{self.base_url}/api/ng/superadmin/users/"
            f"{self.ui_credentials.username}/organizations"
        )
        params = {
            "includeDefaultOrgs": True,
        }
        try:
            response = self.session.get(url, params=params)
            if response.status_code == 200:
                data = response.json()
                if data.get("success"):
                    print(data.get("messages")[0])
                    for organization in data.get("organizations"):
                        if organization.get("is_superadmin"):
                            return organization.get("organization_uuid")
                else:
                    print(data.get("messages")[0])
                    sys.exit(1)
            else:
                response.raise_for_status()
            return ""
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)

    def get_superadmin_api_key(self) -> str:
        """
        Get the SuperAdmin API key for the user
        """
        url = (
            f"{self.base_url}/api/ng/superadmin/users/"
            f"{self.organization_uuid}/keys/apikey"
        )
        try:
            response = self.session.get(url)
            if response.status_code == 200:
                data = response.json()
                return data.get("api_key")
            else:
                response.raise_for_status()
            return ""
        except requests.exceptions.HTTPError as e:
            print(e)
            sys.exit(1)



import json

from pathlib import Path

import typer


class ContrastUICredentials:
    base_url: str
    username: str
    superadmin: bool
    password: str
    code: str

    def get_username_from_user_input(self) -> None:
        username = typer.prompt("Email")
        # TODO: Validate username/email
        self.username = username

    def get_password_from_user_input(self) -> None:
        password = typer.prompt("Password", hide_input=True)
        # TODO: Validate password
        self.password = password

    def get_code_from_user_input(self) -> None:
        code = typer.prompt("Code")
        # TODO: Validate code
        self.code = code


class ContrastAPICredentials:
    profile: str
    base_url: str
    username: str
    api_key: str
    service_key: str
    organization_uuid: str
    superadmin: bool

    def get_credentials_from_file(self, profile: str) -> None:
        """
        Get credentials from file in JSON format
        """
        credentials_file_path = Path("~/.c6t/credentials.json").expanduser()
        with open(credentials_file_path, "r") as credentials_file:
            credentials = json.load(credentials_file)
        self.profile = profile
        self.base_url = credentials.get(profile).get("url")
        self.username = credentials.get(profile).get("user_name")
        self.api_key = credentials.get(profile).get("api_key")
        self.service_key = credentials.get(profile).get("service_key")
        self.organization_uuid = credentials.get(profile).get("organization_id")
        self.superadmin = credentials.get(profile).get("superadmin")

    def write_credentials_to_file(self) -> None:
        """
        Write credentials to file in JSON format
        """
        credentials = {
            self.profile: {
                "url": self.base_url,
                "user_name": self.username,
                "api_key": self.api_key,
                "service_key": self.service_key,
                "organization_id": self.organization_uuid,
                "superadmin": self.superadmin,
            }
        }
        credentials_file_path = Path("~/.c6t/credentials.json").expanduser()
        credentials_file_path.parent.mkdir(parents=True, exist_ok=True)
        with open(credentials_file_path, "w") as credentials_file:
            json.dump(credentials, credentials_file, indent=4)
